import pygtk
pygtk.require("2.0")
import gtk
import gtk.glade
import commandsplus
from Observable import Observer
from threading import Thread,currentThread
import os
import sys
from Alerts import ErrorAlert
import re
import pickle
import gobject
import time
from ControllableThread import ControllableThread

import logging
logging.basicConfig()

def get_shared_path():
	testfile = 'palm-usage-manager.glade'
	sharedirs = [".",os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]),"../../share/palm-usage-manager"))]
	sharepath = None
	for sharedir in sharedirs:
		fname = os.path.join(os.path.abspath(sharedir),testfile)
		if os.path.exists(fname):
			sharepath = os.path.abspath(sharedir)
			break
	
	if sharepath is None:
		raise Exception, "Palm usage manager shared files " + testfile + " cannot be found in any of " + str(sharedirs) + " default paths"
	
	return sharepath

def get_bin_path():
	testfile = 'kill-palm-ppp'
	sharedirs = [".",os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]),"../../bin"))]
	sharepath = None
	for sharedir in sharedirs:
		fname = os.path.join(os.path.abspath(sharedir),testfile)
		if os.path.exists(fname):
			sharepath = os.path.abspath(sharedir)
			break
	
	if sharepath is None:
		raise Exception, "Palm usage manager scripts " + testfile + " cannot be found in any of " + str(sharedirs) + " default paths"
	
	return sharepath

class Application(gtk.glade.XML,Observer):

	def __init__(self):
		gtk.glade.XML.__init__(self,os.path.join(get_shared_path(),"palm-usage-manager.glade"))
		self.mainThread = currentThread()
		for w in ["usage_manager","network_settings","mode_evolution","mode_ppp","invoke_hotsync_settings","invoke_network_settings","invoke_quit","invoke_minimize","device_file_name","status_message"]:
			setattr(self,w,self.get_widget(w))
		self.signal_autoconnect(self)
		self.usage_manager.show()
		self.load_config()
		
	def quit(self,w=None,e=None):
		print "quitting"
		self.kill_gpilotd()
		print "gpilotd"
		self.kill_pppd()
		print "pppd"
		self.save_config()
		print "save config"
		gtk.main_quit()
		print "gtkmainquit"

	def minimize(self,w=None,e=None):
		self.usage_manager.iconify()

	def kill_gpilotd(self):
		# FIXME we need to kill the precise PID of gpilotd that was launched here, not issue killalls randomly
		p = commandsplus.getstatusoutputerror("killall -INT gpilotd")
		print p
	def kill_pppd(self):
		if hasattr(self,"pppd_thread"):
			self.pppd_thread.kill(asynchronous=True)
			delattr(self,"pppd_thread")
		# FIXME we need to implement a way to kill the pid of the pppd ONLY
		if self.system_is_setup_for_ppp():
			killpppcmd = os.path.join(get_bin_path(),"kill-palm-ppp")
			print commandsplus.getstatusoutputerror("sudo %s"%commandsplus.shell_escape(killpppcmd))
	def run_gpilotd(self):
		def run_and_print(func,*args): print func(*args)
		t = Thread(target=run_and_print,args=(commandsplus.getstatusoutputerror,"/usr/libexec/gpilotd",))
		t.setDaemon(True)
		t.start()
		self.print_status("Evolution HotSync service started")
	def run_pppd(self):
		
		device_file = self.device_file_name.get_text()
		
		class PPPRunner(ControllableThread):
			
			def run(thethread):
				
				def is_there(file):
					try: statstruct = os.stat(file)
					except OSError,e:
						if e.errno == 2: return False
						else: raise
					return True
				
				while not thethread.killed():
					if not is_there(device_file):
						self.print_status("Waiting for incoming PPP connection from Palm...")
						time.sleep(1)
						continue
					self.print_status("Starting PPP service...")
					cmd = "sudo %s %s"%(commandsplus.shell_escape(os.path.join(get_bin_path(),"palm-ppp")),commandsplus.shell_escape(device_file))
					status,output,error = commandsplus.getstatusoutputerror(cmd)
					print status, output, error
					self.print_status("PPP connection terminated: %s"%status.return_value)
		
		self.pppd_thread = PPPRunner()
		self.pppd_thread.setDaemon(True)
		self.pppd_thread.start()

	def print_status(self,message):
		if currentThread() != self.mainThread:
			gtk.threads_enter()
			gobject.idle_add(self.print_status,message)
			gtk.threads_leave()
		
		self.status_message.set_text(message)
		
		#dbuffer = self.status_textview.get_buffer()
		#if dbuffer.get_char_count():
			#message = "\n"+message
		#dbuffer.insert(dbuffer.get_end_iter(),message)
		#self.status_textview.scroll_to_iter(dbuffer.get_end_iter(),0.0)
	
	def system_is_setup_for_ppp(self):
		try: login = os.getlogin()
		except OSError: login = os.getenv("USER")
		if not login: login = os.getenv("LOGNAME")
		exp = "^%s\s*ALL\s*=\s*NOPASSWD:\s*%s$"%(login,os.path.join(get_bin_path(),"palm-ppp"))
		exp2 = "^%s\s*ALL\s*=\s*NOPASSWD:\s*%s$"%(login,os.path.join(get_bin_path(),"kill-palm-ppp"))
		t = file("/etc/sudoers").read(-1)
		match = re.findall(exp,t,re.MULTILINE)
		match2 = re.findall(exp2,t,re.MULTILINE)
		#return False
		return match and match2
	
	def load_config(self):
		fn = os.path.expanduser("~/.palm-usage-managerrc")
		try:
			f = file(fn)
			self.settings = pickle.load(f)
		except IOError:
			self.settings = {}
		self.apply_config()
		
	def apply_config(self):
		if self.settings.has_key("pppdevice"):
			self.device_file_name.set_text(self.settings["pppdevice"])
		if self.settings.has_key("mode"):
			if self.settings["mode"] == "evolution":
				self.toggle_mode() # since it is the defalt, we just need to run the toggle_mode code
			elif self.settings["mode"] == "ppp":
				self.mode_ppp.set_active(True)
			else: raise AssertionError, "NotReached"
		else:
			self.toggle_mode() # run with evol default

	def gather_config(self):
		self.settings["pppdevice"] = self.device_file_name.get_text()
		self.settings["mode"] = self.mode
		
	def save_config(self):
		self.gather_config()
		fn = os.path.expanduser("~/.palm-usage-managerrc")
		try:
			f = file(fn,"wb")
			pickle.dump(self.settings,f)
		except IOError: pass
	
	def toggle_mode(self,w=None,e=None):
		
		if self.mode_evolution.get_active(): self.mode = "evolution"
		elif self.mode_ppp.get_active(): self.mode = "ppp"
		else: raise AssertionError,"NotReached"
		
		#we would do very well to encapsualte this functionality in another module
		
		if self.mode == "evolution":
			self.kill_gpilotd()
			self.kill_pppd()
			self.run_gpilotd()
		elif self.mode == "ppp":
			self.kill_gpilotd()
			self.kill_pppd()
			
			if self.system_is_setup_for_ppp():
				self.run_pppd()
			else:
				try: login = os.getlogin()
				except OSError: login = os.getenv("USER")
				if not login: login = os.getenv("LOGNAME")
				print login
				#fixme lets make the command accept spaces by escaping them, and warn the user if the path contains a double or single quote, or maybe escape them as well
				pppcmd = os.path.join(get_bin_path(),"palm-ppp")
				killpppcmd = os.path.join(get_bin_path(),"kill-palm-ppp")
				message = """Prior to using this feature, you need to let your computer run palm-ppp with administrator privileges.  To accomplish this, place the following in your /etc/sudoers file:

%s	ALL = NOPASSWD: %s
%s	ALL = NOPASSWD: %s"""%(login,pppcmd,login,killpppcmd)
				d = ErrorAlert(self.usage_manager,"Your computer is not set up yet",message)
				d.run()
				self.mode_evolution.set_active(True)
				
		else: raise AssertionError, "NotReached"

	def show_network_settings(self,w=None,e=None):
		self.network_settings.show()

	def show_hotsync_settings(self,w=None,e=None):
		def run_and_print(func,*args): print func(*args)
		cmd = "gpilotd-control-applet"
		Thread(target=run_and_print,args=(commandsplus.getstatusoutputerror,cmd)).start()

	def network_settings_closed(self,w=None,e=None):
		self.network_settings.hide()
		if self.mode == "ppp":
			self.kill_pppd()
			self.run_pppd()
		return True

def test():
	gtk.threads_init()
	app = Application()
	gtk.main()
	print "finished"
	
if __name__ == "__main__":
	test()